/*================================================================
 sngtc_cfg.c


*/


#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/if.h>
#include <arpa/inet.h>
#include <errno.h>
#include <sng_tc/sng_tc.h>

#if 0
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#endif

enum {
	SNGTC_LOGLEVEL_L_INFO = SNGTC_LOGLEVEL_PUBLIC,
	SNGTC_LOGLEVEL_L_RES
};


#define SNGTC_PRINT(level,a,...) log_printf(level,a,##__VA_ARGS__)
#define SNGTC_DEBUG(a,...) 		SNGTC_PRINT(SNGTC_LOGLEVEL_DEBUG,a,##__VA_ARGS__)
#define SNGTC_WARN(a,...) 		SNGTC_PRINT(SNGTC_LOGLEVEL_WARN,a,##__VA_ARGS__)
#define SNGTC_INFO(a,...) 		SNGTC_PRINT(SNGTC_LOGLEVEL_L_INFO,a,##__VA_ARGS__)
#define SNGTC_STATS(a,...) 		SNGTC_PRINT(SNGTC_LOGLEVEL_STATS,a,##__VA_ARGS__)
#define SNGTC_ERROR(a,...)		SNGTC_PRINT(SNGTC_LOGLEVEL_ERROR,a,##__VA_ARGS__)
#define SNGTC_CRIT(a,...) 		SNGTC_PRINT(SNGTC_LOGLEVEL_CRIT,a,##__VA_ARGS__)
#define SNGTC_RES(a,...) 		SNGTC_PRINT(SNGTC_LOGLEVEL_L_RES,a,##__VA_ARGS__)
#define SNGTC_TEST(a,...)

#define SNGTC_L_LOGLEVEL_DECODE(level) \
		((level==SNGTC_LOGLEVEL_DEBUG) ? "SNGTC_DEBUG" : \
		(level==SNGTC_LOGLEVEL_WARN) ? "SNGTC_WARN" : \
		(level==SNGTC_LOGLEVEL_INFO) ? "SNGTC_INFO" : \
		(level==SNGTC_LOGLEVEL_L_INFO) ? "SNGTC_INFO" : \
		(level==SNGTC_LOGLEVEL_L_RES) ? "SNGTC_RESLT" : \
		(level==SNGTC_LOGLEVEL_STATS) ? "SNGTC_STATS" : \
		(level==SNGTC_LOGLEVEL_ERROR) ? "SNGTC_ERROR" : \
		(level==SNGTC_LOGLEVEL_CRIT) ? "SNGTC_CRIT" : "Unkown")


typedef struct {

	int init;
	sngtc_codec_request_t codec_request;
	sngtc_codec_reply_t codec_reply;
	sngtc_rtp_stats_t rtp_stats;
 	int index;
	/* Other private info */

}private_codec_t;

typedef struct system_eth_cfg{
	int 			init;
	char 			name[100];
	sngtc_eth_mac_t mac;
	sngtc_ip_t 		ip;
	sngtc_ip_t 		mask;
	uint32_t 		flags;
	uint8_t 		set_if_up;
}system_eth_cfg_t;

#define MAX_ETH_DEVS 32
system_eth_cfg_t sys_cfg[MAX_ETH_DEVS];
uint32_t sys_cfg_sz=0;

#define MAX_CODEC_IDX_SZ 5000
#define MAX_RTP_SESSIONS 1
private_codec_t g_codec_idx[MAX_CODEC_IDX_SZ];
uint32_t g_codec_idx_cnt=0;
static int verbose=0;
static int asterisk=0;
static int freeswitch=0;
static int user_color=0;

void textcolor(int attr, int fg, int bg)
{	char command[13];

	/* Command is the control command to the terminal */
	sprintf(command, "%c[%d;%d;%dm", 0x1B, attr, fg + 30, bg + 40);
	printf("%s", command);
}

#define SNG_RED "\033[22;31m"
#define SNG_BLACK "\033[22;30m"
#define SNG_GREEN "\033[22;32m"
enum {
	SNG_COLOR_BLACK,
	SNG_COLOR_RED,
	SNG_COLOR_GREEN
};

int log_printf(int level, char *fmt, ...)
{
	char *data;
	int ret = 0;
	va_list ap;
	char date[200] = "";

	struct tm now;
	time_t epoch;

	switch (level) {
	case SNGTC_LOGLEVEL_DEBUG:
		if (verbose < 10) {
			return -1;
		}
	case SNGTC_LOGLEVEL_STATS:
	case SNGTC_LOGLEVEL_WARN:
	case SNGTC_LOGLEVEL_INFO:
		if (verbose == 0) {
			return -1;
		}
	case SNGTC_LOGLEVEL_L_INFO:
	case SNGTC_LOGLEVEL_L_RES:
	case SNGTC_LOGLEVEL_ERROR:
	case SNGTC_LOGLEVEL_CRIT:
		break;
	default:
		fprintf(stderr, "%s: invalid loglevel %i\n",__FUNCTION__,level);
		return -1;
	}

	if (time(&epoch) && localtime_r(&epoch, &now)) {
		strftime(date, sizeof(date), "%Y-%m-%d %T", &now);
	}

	va_start(ap, fmt);
	ret = vasprintf(&data, fmt, ap);
	if (ret == -1) {
		return ret;
	}
        
	if (!user_color) {
		printf("[%-11s] %s", SNGTC_L_LOGLEVEL_DECODE(level), data);
	} else {
		switch (level) {
		case SNGTC_LOGLEVEL_ERROR:
		case SNGTC_LOGLEVEL_CRIT:
			printf(SNG_RED "[%-11s] %s", SNGTC_L_LOGLEVEL_DECODE(level), data);
			break;
		case SNGTC_LOGLEVEL_L_RES:
			printf(SNG_GREEN "[%-11s] %s", SNGTC_L_LOGLEVEL_DECODE(level), data);
			break;
		default:
			printf(SNG_BLACK "[%-11s] %s", SNGTC_L_LOGLEVEL_DECODE(level), data);
			break;
		}
	}

	return 0;
}

#define PROD "sngtc_cfg"


int sngtc_detect_eth_dev(void)
{

  	static struct ifreq ifreqs[20];
   	struct ifconf ifconf;
	struct ifreq  ifr;
   	int  nifaces, i;
	int rval;
	int sock;
	
	sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0) {
        return -1;
    }


	memset(&ifr,0,sizeof(ifr));
	memset(&ifconf,0,sizeof(ifconf));
	ifconf.ifc_buf = (char*) (ifreqs);
	ifconf.ifc_len = sizeof(ifreqs);

	if ((rval = ioctl(sock, SIOCGIFCONF , (char*) &ifconf  )) < 0 ) {
		perror("ioctl(SIOGIFCONF)");
		return -1;
	}
	
	nifaces =  ifconf.ifc_len/sizeof(struct ifreq);

	SNGTC_INFO("====================================================================\n");
	SNGTC_INFO("\n");
	SNGTC_INFO("Detected Local Ethernet Interfaces:\n");

	for(i = 0; i < nifaces && i < MAX_ETH_DEVS; i++) {

		SNGTC_DEBUG("\t%-10s\n", ifreqs[i].ifr_name);
		memset(&ifr,0,sizeof(ifr));
		strcpy(ifr.ifr_name, ifreqs[i].ifr_name);

		if (ioctl(sock, SIOCGIFFLAGS, &ifr) == 0) {
            if (! (ifr.ifr_flags & IFF_LOOPBACK)) {

				memset(&sys_cfg[sys_cfg_sz],0,sizeof(sys_cfg[sys_cfg_sz]));

				sys_cfg[sys_cfg_sz].init=1;
				
				snprintf(sys_cfg[sys_cfg_sz].name, sizeof(sys_cfg[sys_cfg_sz].name), "%s",ifreqs[i].ifr_name);
				sys_cfg[sys_cfg_sz].flags=ifr.ifr_flags;


				if (!(ifr.ifr_flags & IFF_UP)) {

					memset(&ifr,0,sizeof(ifr));
					strcpy(ifr.ifr_name, ifreqs[i].ifr_name);
					ifr.ifr_flags=sys_cfg[sys_cfg_sz].flags|IFF_UP|IFF_RUNNING;
					if (ioctl(sock, SIOCSIFFLAGS, &ifr) != 0) {
						SNGTC_ERROR("IF=%s SIOCSIFFLAGS: %s\n",ifreqs[i].ifr_name,strerror(errno));
						continue;
					}

					sys_cfg[sys_cfg_sz].set_if_up=1;
					SNGTC_INFO("Bringing up %s \n",ifreqs[i].ifr_name);
				}

				memset(&ifr,0,sizeof(ifr));
				strcpy(ifr.ifr_name, ifreqs[i].ifr_name);
                if (ioctl(sock, SIOCGIFHWADDR, &ifr) != 0) {
					SNGTC_ERROR("IF=%s SIOCGIFHWADDR: %s\n",ifreqs[i].ifr_name,strerror(errno));
					continue;
				}

				memcpy(sys_cfg[sys_cfg_sz].mac.mac,ifr.ifr_hwaddr.sa_data,sizeof(sys_cfg[sys_cfg_sz].mac));
				sngtc_mac_addr_to_str(sys_cfg[sys_cfg_sz].mac.mac_str,sys_cfg[sys_cfg_sz].mac.mac);
				

				memset(&ifr,0,sizeof(ifr));
				strcpy(ifr.ifr_name, ifreqs[i].ifr_name);
				if (ioctl(sock, SIOCGIFADDR, &ifr) != 0) {
					SNGTC_ERROR("IF=%s SIOCGIFADDR: %s\n",ifreqs[i].ifr_name,strerror(errno));
					continue;
				}

				sys_cfg[sys_cfg_sz].ip.ip=ntohl(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr);
				sngtc_codec_ipv4_hex_to_str(sys_cfg[sys_cfg_sz].ip.ip,sys_cfg[sys_cfg_sz].ip.ip_str);
	
				memset(&ifr,0,sizeof(ifr));
				strcpy(ifr.ifr_name, ifreqs[i].ifr_name);
				if (ioctl(sock, SIOCGIFNETMASK, &ifr) != 0) {
					SNGTC_ERROR("IF=%s SIOCGIFNETMASK: %s\n",ifreqs[i].ifr_name,strerror(errno));
					continue;
				}

				sys_cfg[sys_cfg_sz].mask.ip=ntohl(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr);
				sngtc_codec_ipv4_hex_to_str(sys_cfg[sys_cfg_sz].mask.ip,sys_cfg[sys_cfg_sz].mask.ip_str);

				SNGTC_INFO("%s: MAC=%s IP=%s/%s\n",
					sys_cfg[sys_cfg_sz].name, sys_cfg[sys_cfg_sz].mac.mac_str,sys_cfg[sys_cfg_sz].ip.ip_str,sys_cfg[sys_cfg_sz].mask.ip_str);

				sys_cfg_sz++;
            }
        }
		
	}

	if (sys_cfg_sz == 0) {
		SNGTC_ERROR("\tNo Local Ethernet Intefaces Found!\n");
	}
	SNGTC_INFO("\n");
	SNGTC_INFO("====================================================================\n");

	return sys_cfg_sz;
}

int sngtc_find_local_eth_by_mac(uint8_t *mac, system_eth_cfg_t **eth_cfg)
{
	int i;

	*eth_cfg=NULL;

	for (i=0;i<sys_cfg_sz;i++) {

		if (!sys_cfg[i].init) {
			continue;
		}

		if (memcmp(sys_cfg[i].mac.mac,mac,sizeof(sys_cfg[i].mac.mac)) == 0) {
			*eth_cfg = &sys_cfg[i];
			return 0;
		}
	}

	return -1;
}

#define SNGTC_ASTERISK_CFG  	"sangoma_codec.conf"
#define SNGTC_ASTERISK_DIR  	"/etc/asterisk"
#define SNGTC_FREESWITCH_CFG  	"sangoma_codec.conf.xml"
#define SNGTC_FREESWITCH_DIR	"/usr/local/freeswitch/conf/autoload_configs"

int sngtc_create_cfg(FILE **fd, char *file_name)
{
	*fd = fopen(file_name, "w");
	if (!*fd) {
		SNGTC_CRIT("Failed to write codec cfg file %s!\n",
			file_name);
		return -1;
	}
	return 0;
}

int sngtc_write_freeswich_file_begin(FILE *fd)
{
	fprintf(fd,"<configuration name=\"sangoma_codec.conf\" description=\"Sangoma Codec Configuration\">\n\n");

	fprintf(fd,"\t<settings>\n"
		"\t\t<!--\n"
		"\t\tComma separated list of codecs to register with FreeSWITCH,\n"  
		"\t\tby default (if this parameter is not set) all available codecs are registered.\n" 
		"\t\tValid codec values are: PCMU,PCMA,L16,G729,G726-32,GSM\n" 
		"\t\t<param name=\"load\" value=\"all\"/>\n" 
		"\t\t-->\n" 

		"\t\t<!-- List of codecs to not register with FreeSWITCH, by default this is empty,\n" 
	    "\t\t    but you may want to not load PCMU and PCMA or may be others to not use your\n"  
	    "\t\t    vocallo resources in codecs that are done well and fast in software.\n" 	
		"\t\t-->\n" 
		"\t\t<param name=\"noload\" value=\"PCMU,PCMA,L16\"/> \n" 
		);

	fprintf(fd,"\t</settings>\n\n");

	fprintf(fd,"\t<vocallos>\n\n");
	return 0;
}

int sngtc_write_freeswich_file_end(FILE *fd)
{

	fprintf(fd,"\t</vocallos>\n\n");

	fprintf(fd,"</configuration>\n\n");

	return 0;
}



int sntc_write_freeswitch_file(FILE *fd, system_eth_cfg_t *e_cfg, sngtc_nic_vocallo_map_t *nic_map)
{
	
	fprintf(fd,"\t\t<vocallo name=\"%s\">\n\n"
	           "\t\t\t<!-- Starting UDP port for the vocallo -->\n"
			   "\t\t\t<param name=\"baseudp\" value=\"%d\"/>\n"
			   "\t\t\t<!-- Starting IP address to use for the vocallo modules -->\n"
			   "\t\t\t<param name=\"vocalloaddr\" value=\"%s\"/>\n\n"
			   "\t\t</vocallo>\n\n",
				e_cfg->name, nic_map->voc_base_udp_prot, nic_map->voc_base_ip.ip_str);
	return 0;
}

int sngtc_write_asterisk_file_begin(FILE *fd)
{
	
	fprintf(fd, "[general]\n\n"
				";Attempt to avoid codec conflicts in Asterisk that can cause Asterisk\n"
				";to crash on codec module load\n");
	fprintf(fd, "unregisterconflictingcodecs=yes\n\n");
	return 0;
}

int sntc_write_asterisk_file(FILE *fd, system_eth_cfg_t *e_cfg, sngtc_nic_vocallo_map_t *nic_map)
{
	fprintf(fd, "[%s]\n\n", e_cfg->name);

	fprintf(fd, ";Base udp port to be used by the sangoma vocallo modules, if not specified, some default will be used\n"
			    ";this is NOT the base UDP to be used by Asterisk, the udp port to be used by Asterisk will be set\n"
			    ";by the rtp stack ( whatever RTP stack Asterisk uses)\n");

	fprintf(fd, "baseudp=%d\n\n",nic_map->voc_base_udp_prot);

	fprintf(fd, ";Starting IP address of vocallo modules\n");
	fprintf(fd, "vocalloaddr=%s\n\n",nic_map->voc_base_ip.ip_str);

	return 0;
}

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*\

Function:       main

Description:    Main program.

\*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/



int main(int argc, char *argv[])
{
	int i,x;
	int err;
	char res_str[1024];
	int res_int=0;
	int  ip_nibble;
	sngtc_init_cfg_t init_cfg;
	system_eth_cfg_t *eth_cfg;
	FILE *fd;
	char file_name_tmp[1000];
	char file_name[1000];
	char cfg_dir[500];
	char in[1024];
	int  got_cfgdir=0;
	int  dev_cfg=0;
	int  detect=0;
	int  detect_cnt=0;

	sngtc_nic_vocallo_map_t *nic_voc_map;
	sngtc_vocallo_map_t *voc_map;
	int	 nic_voc_map_sz;
	
	for (i=0; i<argc;i++) {
		if (!strcmp(argv[i],"verbose")) {
			if (i+1 >= argc) {
				break;
			}
			verbose = atoi(argv[i+1]);
		}
		if (!strcmp(argv[i],"asterisk")) {
			asterisk = 1;
		}
		if (!strcmp(argv[i],"freeswitch")) {
			freeswitch = 1;
		}
		if (!strcmp(argv[i],"cfg_dir")) {
			if (i+1 >= argc) {
				break;
			}
			snprintf(cfg_dir,sizeof(cfg_dir),argv[i+1]);
			got_cfgdir=1;
		}
		if (!strcmp(argv[i], "color")) {
			if (i+1 >= argc) {
				break;
			}
			user_color=1;
		}
	}

	if (asterisk) {
		if (!got_cfgdir) {
			snprintf(cfg_dir,sizeof(cfg_dir),SNGTC_ASTERISK_DIR);
		}
		snprintf(file_name,sizeof(file_name),"%s/%s",cfg_dir,SNGTC_ASTERISK_CFG);
		snprintf(file_name_tmp,sizeof(file_name),"%s/%s.tmp",cfg_dir,SNGTC_ASTERISK_CFG);
	} else if (freeswitch) {
		if (!got_cfgdir) {
			snprintf(cfg_dir,sizeof(cfg_dir),SNGTC_FREESWITCH_DIR);
		}
		snprintf(file_name,sizeof(file_name),"%s/%s",cfg_dir,SNGTC_FREESWITCH_CFG);
		snprintf(file_name_tmp,sizeof(file_name),"%s/%s.tmp",cfg_dir,SNGTC_FREESWITCH_CFG);

	} else {
		SNGTC_INFO("\n");
		SNGTC_ERROR("Invalid arguments!\n");
		SNGTC_INFO("Usage: sngtc_cfg [asterisk|freeswitch] <verbose [0-100]> <color [0|1]> \n");
		SNGTC_INFO("\n");
		return -1;
	}

	system("clear");

	SNGTC_INFO("\n");
	SNGTC_INFO("====================================================================\n");
	if (asterisk) {
		SNGTC_INFO("Configuring D100 for Asterisk\n");
	} else {
		SNGTC_INFO("Configuring D100 for FreeSwitch\n");
	}
	SNGTC_INFO("\n");

    /*
     * Display version information.
     */	


	err=sngtc_detect_eth_dev();
	if (!err) {
		SNGTC_INFO("\n");
		return -1;
	}


	SNGTC_DEBUG("%s: Detected %i eth devices\n",PROD,err);

	memset(&init_cfg,0,sizeof(init_cfg));
	init_cfg.log = log_printf;

	SNGTC_INFO("\n");
	SNGTC_INFO("Detecting D100 Card & Modules ...\n",
			nic_voc_map_sz);

	if (!detect) {
		err=sngtc_get_nic_vocallo_map(&init_cfg,&nic_voc_map,&nic_voc_map_sz);
		if (err) {
			SNGTC_ERROR("%s: Failed to detect D100 transcoding modules !\n",PROD);
			SNGTC_INFO("\n");
			return err;
		}
	}

	err=sngtc_create_cfg(&fd,file_name_tmp);
	if (err) {
		SNGTC_INFO("\n");
		return err;
	}

	if (asterisk) {
		sngtc_write_asterisk_file_begin(fd);
	}

	if (freeswitch) {
		sngtc_write_freeswich_file_begin(fd);
	}

retry_again:

	detect++;

	SNGTC_INFO("\n");
	SNGTC_INFO("Detected D100 Card & Modules\n",
			nic_voc_map_sz);
	SNGTC_INFO("\n");


	detect_cnt=0;

	for (i=0;i<nic_voc_map_sz;i++) {

		detect_cnt++;

		if (nic_voc_map[i].init > 1) {
			continue;
		}

		err=sngtc_find_local_eth_by_mac(nic_voc_map[i].nic_mac.mac, &eth_cfg);
		if (err) {
			SNGTC_ERROR("%s: Failed to detect eth device with mac %s\n",
					PROD,nic_voc_map[i].nic_mac.mac_str);
			continue;
		}

		SNGTC_INFO("[%d]: %s: Mac=%s IP=%s/%s\n",
				 i+1,
				 eth_cfg->name,
				 nic_voc_map[i].nic_mac.mac_str,
				 eth_cfg->ip.ip_str,
				 eth_cfg->mask.ip_str);

		SNGTC_INFO("--------------------------------------------------------\n");
		if (nic_voc_map[i].voc_mods_sz) {

			for (x=0; x<nic_voc_map[i].voc_mods_sz; x++) {
				voc_map = &nic_voc_map[i].voc_map[x];
				SNGTC_INFO("\t-> D100-Module %i: Mac=%s Error=%i\n",
						x+1,voc_map->v_mac.mac_str,voc_map->error);
			}
			if (i!=nic_voc_map_sz) {
				SNGTC_INFO("\n");
			}
		}


	}
         
	SNGTC_INFO("--------------------------------------------------------\n");
	SNGTC_INFO("\n");
	SNGTC_INFO("Please select device to configure: [1-%i] q:quit ",detect_cnt);

	fgets(in,sizeof(in),stdin);
	sscanf(in,"%s",res_str);
	if (in[0] == 'q') {
		goto done;
	}

    i=atoi(in);
	
	if (i > 0 && i <= detect_cnt) {

			i--;
			
			err=sngtc_find_local_eth_by_mac(nic_voc_map[i].nic_mac.mac, &eth_cfg);
			if (err || !eth_cfg) {
				SNGTC_CRIT("%s: Failed to detect eth device with mac %s - should never happen\n",
						PROD,nic_voc_map[i].nic_mac.mac_str);
				/* we cannot recover here because this should never fail */
				exit(1);
			}

			if (nic_voc_map[i].init > 1) {
				SNGTC_ERROR("Device %s already configured!\n",
					eth_cfg->name);
				goto retry_again;
			}
			nic_voc_map[i].init++;
			dev_cfg++;

			SNGTC_INFO("--------------------------------------------------------\n");
	   	 	SNGTC_INFO("Configuring %s: Mac=%s IP=%s/%s %p\n",
				 eth_cfg->name,
				 nic_voc_map[i].nic_mac.mac_str,
				 eth_cfg->ip.ip_str,
				 eth_cfg->mask.ip_str,
				 &nic_voc_map[i]);
	

			nic_voc_map[i].voc_base_ip.ip = eth_cfg->ip.ip;
			nic_voc_map[i].voc_base_ip.ip &= 0xFFFFFF00;
			nic_voc_map[i].voc_base_ip.ip |= 0x00000064;
			sngtc_codec_ipv4_hex_to_str(nic_voc_map[i].voc_base_ip.ip,nic_voc_map[i].voc_base_ip.ip_str);

			nic_voc_map[i].voc_base_netmask = eth_cfg->mask;

retry:
			SNGTC_INFO("\n");
			SNGTC_INFO("\t----------------------------------------------------------\n");
			SNGTC_INFO("\tPlease specify the base IP/Netmask range for D100 Modules:\n");
			SNGTC_INFO("\tDefault ip/netmask value  : %s/%s \n",
						nic_voc_map[i].voc_base_ip.ip_str,
						nic_voc_map[i].voc_base_netmask.ip_str);
			SNGTC_INFO("\n");
			SNGTC_INFO("\tSpecify last nibble of the ip addres: [1-255] or <enter> for default:\n");
			SNGTC_INFO("\t#> ");
			memset(res_str,0,sizeof(res_str));

			fgets(in,sizeof(in),stdin);
			sscanf(in,"%s",res_str);

			if ((ip_nibble=atoi(res_str)) > 0) {
				int orig_nibble = eth_cfg->ip.ip & 0xFF;
				SNGTC_DEBUG("\tIP Nibble %i\n",orig_nibble);
				if (orig_nibble != ip_nibble &&
                    orig_nibble > ip_nibble &&
					orig_nibble-nic_voc_map[i].voc_mods_sz > 0 &&
					orig_nibble-nic_voc_map[i].voc_mods_sz >= ip_nibble) {
					nic_voc_map[i].voc_base_ip.ip &= 0xFFFFFF00;
					nic_voc_map[i].voc_base_ip.ip |= ip_nibble;
					sngtc_codec_ipv4_hex_to_str(nic_voc_map[i].voc_base_ip.ip,nic_voc_map[i].voc_base_ip.ip_str);
				} else {
					SNGTC_ERROR("\tSelected D100 Module IP/Mask = %s/%s\n",
							eth_cfg->ip.ip_str,
							nic_voc_map[i].voc_base_netmask.ip_str);
					SNGTC_ERROR("\tD100 Module Base IP cannot be the same as or within range of %s IP %s\n",
						eth_cfg->name,eth_cfg->ip.ip_str);
					goto retry;
				}
			} 

			nic_voc_map[i].voc_base_udp_prot = 15000;

			SNGTC_RES("\tSelected D100 Module IP/Mask = %s/%s \n",
				nic_voc_map[i].voc_base_ip.ip_str,
				nic_voc_map[i].voc_base_netmask.ip_str);

			SNGTC_INFO("\n");
			SNGTC_INFO("\t----------------------------------------------------------\n");
			SNGTC_INFO("\tPlease specify the base UDP port for D100 Modules:\n");
			SNGTC_INFO("\tDefault udp base value  : %d \n",
						nic_voc_map[i].voc_base_udp_prot);
			SNGTC_INFO("\n");
			SNGTC_INFO("\tSpecify udp base value [9000-65000] or <enter> for default:\n");
			SNGTC_INFO("\t#> ");
			
			fgets(in,sizeof(in),stdin);
			sscanf(in,"%s",res_str); 

			if (res_int >=9000) {
				nic_voc_map[i].voc_base_udp_prot=res_int;
			}
			SNGTC_RES("\tSelected UDP Port Base %d\n",
						nic_voc_map[i].voc_base_udp_prot);
			SNGTC_INFO("\n");
			SNGTC_INFO("====================================================================\n");
			SNGTC_INFO("\n");

			if (asterisk) {
				err=sntc_write_asterisk_file(fd,eth_cfg,&nic_voc_map[i]);
			} else {
				err=sntc_write_freeswitch_file(fd,eth_cfg,&nic_voc_map[i]);
			}

			if (err) {
				SNGTC_INFO("\n");
				return -1;
			}

		if (dev_cfg != detect_cnt) {
        	goto retry_again;
		}
	} else {
		SNGTC_ERROR("Invalid option selected\n");
     	goto retry_again;
	}

done:
	if (freeswitch) {
		sngtc_write_freeswich_file_end(fd);

	} 

	fclose(fd);

	if (dev_cfg) {
		char cmd[1024];
		SNGTC_INFO("\n");
		SNGTC_INFO("Sangoma Transcoding configuration file saved:\n");
		SNGTC_RES ("\t%s\n",file_name);
		SNGTC_INFO("\n");

		/* This is a hack :) use fread fwrite */
		sprintf(cmd,"mv %s %s\n",file_name_tmp,file_name);
		system(cmd);
	}

	unlink(file_name_tmp);
	

    return 0;
}

